-- mglass.ex: magnifying glass

-- Jiri Babor
-- j.babor@gns.cri.nz

include graphics.e
include machine.e
include image.e

constant
   h=96, w=96,
   mmin=2, mmax=8,      -- magnification limits
   tile={{ 3, 9, 9,12, 5, 5, 5, 5},
         { 3, 9, 9, 4,12,12,12,12},
         { 3, 9, 9, 4,12,12,12,12},
         { 3, 9, 9, 0, 9, 1, 1, 0},
         { 5, 5, 5, 5, 3, 9, 9,12},
         {12,12,12,12, 3, 9, 9, 4},
         {12,12,12,12, 3, 9, 9, 4},
         { 9, 1, 1, 0, 3, 9, 9, 0}},
   block={
         "function poll_mouse()",
         "   -- returns mouse status: {x_coord,y_coord,button_status}",
         "   -- button status: 0 .. no button pressed",
         "   --                1 .. left button pressed",
         "   --                2 .. right button pressed",
         "   --                4 .. middle button pressed",
         "   sequence  rl -- list of register values",
         "   rl = repeat(0,10)",
         "   rl[REG_AX] = 3",
         "   rl = dos_interrupt(#33,rl)",
         "   return {rl[REG_CX], rl[REG_DX], rl[REG_BX]}",
         "end function   -- poll_mouse",
         "",
         "procedure glass()",
         "   integer c,r,x1,x2,y1,y2",
         "   ms=poll_mouse()",
         "   if ms[3] and time()-t>0.2 then",
         "      if and_bits(ms[3],1) then     -- increase magnification",
         "         if m<mmax then m=m+2 mc=1 end if",
         "      elsif and_bits(ms[3],2) then  -- decrease magnification",
         "         if m>mmin then m=m-2 mc=1 end if",
         "      end if"}
object junk
sequence screen,s,ms
atom t
integer m,mc,x,y

procedure box(object c,integer f,sequence s1, sequence s2)
   -- rectangle given by its diagonal points s1,s2
   -- c is color, or 2-dimensional tile sequence
   -- i2 is fill flag (0/1 -> empty/filled)
   integer dx,lx,ly,mx,my,x,y,y2
   sequence cs
   if sequence(c)  then       -- tiled fill
      s1=floor(s1)
      s2=floor(s2)
      dx=s2[1]-s1[1]+1
      x=s1[1]
      y=s1[2]
      y2=s2[2]
      dx=s2[1]-s1[1]
      ly=length(c)
      lx=length(c[1])
      mx=1+remainder(s1[1],lx)
      cs=c
      for i=1 to ly do
         while length(cs[i])<dx+lx do cs[i]=cs[i]&c[i] end while
      end for
      while y<=y2 do
         my=1+remainder(y,ly)
         pixel(cs[my][mx..mx+dx],{x,y})
         y=y+1
      end while
   else
      polygon(c,f,{s1,{s2[1],s1[2]},s2,{s1[1],s2[2]}})
   end if
end procedure

procedure write_block(sequence s1, sequence s2)
   -- write sequence of strings s2 starting at point s1
   for i=1 to length(s2) do
      position(s1[1],s1[2])
      puts(1,s2[i])
      s1[1]=s1[1]+1
   end for
end procedure

procedure set_mouse_xy(sequence s)
   -- places the mouse pointer where you want it
   sequence rl -- list of register values
   rl = repeat(0,10)
   rl[REG_AX] = 4
   rl[REG_CX] = s[1]
   rl[REG_DX] = s[2]
   rl = dos_interrupt(#33,rl)
end procedure  -- set_mouse_xy

procedure confine_mouse(sequence s1,sequence s2)
   -- confines mouse pointer to within input rectangle s1={x1,y1}, s2={x2,y2}
   sequence rl -- list of register values
   rl = repeat(0,10)
   rl[REG_AX] = 7
   rl[REG_CX] = s1[1]
   rl[REG_DX] = s2[1]
   rl = dos_interrupt(#33,rl)
   rl[REG_AX] = 8
   rl[REG_CX] = s1[2]
   rl[REG_DX] = s2[2]
   rl = dos_interrupt(#33,rl)
end procedure  -- confine_mouse

function poll_mouse()
   -- returns mouse status: {x_coord,y_coord,button_status}
   -- button status: 0 .. no button pressed
   --                1 .. left button pressed
   --                2 .. right button pressed
   --                4 .. middle button pressed (if your rat has it)
   sequence  rl -- list of register values
   rl = repeat(0,10)
   rl[REG_AX] = 3
   rl = dos_interrupt(#33,rl)
   return {rl[REG_CX], rl[REG_DX], rl[REG_BX]}
end function   -- poll_mouse

procedure glass()
   integer c,r,x1,x2,y1,y2
   ms=poll_mouse()
   if ms[3] and time()-t>0.2 then
      if and_bits(ms[3],1) then        -- increase magnification
         if m<mmax then m=m+2 mc=1 end if
      elsif and_bits(ms[3],2) then     -- decrease magnification
         if m>mmin then m=m-2 mc=1 end if
      end if
            t=time()
   end if
   if x!=ms[1] or y!=ms[2] or mc then  -- if mouse pointer moved
      mc=0  -- reset magnification change flag
      -- restore background
      if y>ms[2] then
         if x>ms[1] then
            for i=y to ms[2]+h-1 do
               pixel(screen[i+1][ms[1]+w..x+w], {ms[1]+w-1,i})
            end for
         else
            for i=y to ms[2]+h-1 do
               pixel(screen[i+1][x+1..ms[1]+1], {x,i})
            end for
         end if
         for i=ms[2]+h to y+h-1 do
            pixel(screen[i+1][x+1..x+w], {x,i})
         end for
      else  -- y<=ms[2]
         for i=y to ms[2]-1 do
            pixel(screen[i+1][x+1..x+w], {x,i})
         end for
         if x>ms[1] then
            for i=ms[2] to y+h-1 do
               pixel(screen[i+1][ms[1]+w..x+w], {ms[1]+w-1,i})
            end for
         else
            for i=ms[2] to y+h-1 do
               pixel(screen[i+1][x+1..ms[1]+1], {x,i})
            end for
         end if
      end if
      x=ms[1]  y=ms[2]  -- store current pointer coordinates
      -- magnify
      x1=(w-w/m)/2+1
      x2=(w+w/m)/2
      y1=(h-h/m)/2+1
      y2=(h+h/m)/2
      r=1               -- glass row
      for i=y1 to y2 do
         c=1            -- glass column
         for j=x1 to x2 do
            for k=0 to m-1 do
               s[r+k][c..c+m-1]=screen[i+y][j+x]
            end for
            c=c+m
         end for
         r=r+m
      end for
      display_image({x,y},s)
      box(15,0,{x,y},{x+w-1,y+h-1})    -- glass rim
   end if
end procedure


-- main ------------------------------------------------------------------------

--while get_key()=-1 do  end while
junk=graphics_mode(18)

-- generate background
junk=palette(0,{47,47,47})
junk=palette(3,{31,31,63})
junk=palette(5,{63,31,31})
junk=palette(6,{ 0,0,0})
box(tile,1,{0,0},{639,479})
box(0,1,{55,35},{585,410})
box(14,0,{58,38},{582,407})
draw_line(6,{{57,411},{586,411},{586,37}})   -- black shadows
draw_line(6,{{57,412},{587,412},{587,37}})
text_color(4)
write_block({4,10},block)
box(0,1,{55,425},{585,453})
box(15,0,{55,425},{585,453})
draw_line(8,{{56,452},{56,426},{584,426}})      -- grey shadows
draw_line(6,{{57,454},{586,454},{586,427}})     -- black shadows
draw_line(6,{{57,455},{587,455},{587,427}})
text_color(1)
position(28,11)
puts(1,"Press mouse buttons to change magnification, any key to quit.")

screen=save_image({0,0},{639,479})  -- lazy! I hope you have enough memory...
s=repeat(repeat(0,w),h)             -- blank glass
confine_mouse({0,0},{639-w,480-h})
t=time()
m=4                                 -- initial magnification
mc=1                                -- force glass execution
x=0   y=0
set_mouse_xy({x,y})
while get_key()=-1 do glass() end while

junk=graphics_mode(-1)
